# Contributing to Axii

You are welcome to submit merge requests to Axii to add features, improve code quality, or work on 
the module database. Please understand, however, that maintaining Axii requires lots of time. We 
thus ask you to follow the processes outlined in this document in order to make the maintenance 
tractable, plannable, and efficient for the Axii maintainers. 

This means:

- **We will not consider a merge request ready unless it follows our workflows as outlined in this
document.** 
- **We will mark any merge request as draft if it does not comply.**
- **Drafted merge requests will be closed without merging after a longer period of inactivity.**

In the following, our merge request and review process is outlined in detail.


## Before You Work on Axii

- Search for similar issues
- Open an issue if applicable
- Assign labels to the issue that fit the nature of the issue


## When Starting To Work on Axii

- Open a merge request, ideally based on an issue
  - Set yourself as assignee
  - **Do not set any reviewer yet!**
  - Write a descriptive title
  - Mark the merge request as "Draft:" to make clear that it is work in progress
- Update the documentation reflecting your changes


## When You Think Your Contribution Is Finished

- Ensure the documentaton reflects your changes
- Ensure that the CI pipeline passes and the code quality score did not decrease
- Remove the "Draft:" (mark as ready)
- Set descriptive labels to the merge request
  - If you primarily updated modules, set ~"type::module"
  - If you fixed a bug in Axii (i.e., **not** the module database), set ~"type::fix"
  - If you implemented a new feature in Axii (i.e., **not** the module database), set ~"type::feature"
  - If you updated the documentation, set ~"type::docs"
  - If you updated Axii's tool chain, CI, or anything regarding project management, set ~"type::chore"
  - If you refactored code, set ~"type::refactor"
- Write a description explaining the contribution
  -  Reference any related issues or merge requests
- Only **after** all these steps are done, request a review by setting a reviewer (@dreher or @daab)
  - Expect that your merge request can be merged at any point from here on

**Please note:**

- Please keep merge requests small and focused on related subjects
- We will not review any merge request that has not been assigned to us, since we will assume that
  it is still work in progress
- Please avoid pushing any changes after assigning a reviewer


## When You Receive a Review

- Respond to the comments and try to accomodate them
- Resolve threads that you addressed to keep the merge request clean
- Discuss any open questions in the merge request or document decisions made in personal discussions
- Re-request a review after addressing all points
  - Expect that your merge request can be merged at any point from here on


**Thanks for contributing to Axii!**
