#!/usr/bin/env python3

"""App to parse apt requirements."""

# Disable pylint as this file also should by Python2-compatible.
# pylint: skip-file

from __future__ import print_function

import os
import sys


def get_requirements_from_file(requirements_file, found_requirements):
    requirements_path, _ = os.path.split(requirements_file)
    if not os.path.isfile(requirements_file):
        print("File not found: '{}'.".format(requirements_file), file=sys.stderr)
        return
    with open(requirements_file, "r") as file:
        for line in file:
            line = line.strip()
            if line == "" or line.startswith("#"):
                continue
            if line.startswith("-r "):
                _, required_file = line.split(" ")
                required_path = os.path.join(requirements_path, required_file)
                for package in get_requirements_from_file(required_path, found_requirements):
                    yield package
            elif line not in found_requirements:
                found_requirements.append(line)
                yield line


def get_requirements(paths):
    found_requirements = []

    for path in paths:
        for requirement in get_requirements_from_file(path, found_requirements):
            yield requirement


def main(main_script, *requirements):
    if len(requirements) < 1:
        print("Usage: {} [REQUIREMENTS_FILES]".format(main_script), file=sys.stderr)
        sys.exit(1)

    for requirement in get_requirements(requirements):
        print(requirement)


if __name__ == "__main__":
    main(*sys.argv)
