"""Supplies all subcommands available via `axii config`."""

import rich_click as click

from armarx_setup.cli import common
from armarx_setup.cli.config import integration
from armarx_setup.core import error

COMMAND_GROUPS = []  # Default.


@click.group("config", **common.command_kwargs)
@click.argument("variable_name", shell_complete=integration.list_config_options)
@click.argument("variable_value", default=None, required=False,
                shell_complete=integration.list_config_option_values)
@click.option("--global", "-g", "is_global", default=False, is_flag=True, help="Set/get/unset the global variables.")
@click.option("--unset", "-u", default=False, is_flag=True, help="Unsets the variable.")
@click.option("--subconfig", "-s", default=False, is_flag=True, help="Apply to whole subconfig with the name of the "
                                                                     "variable.")
def config(**kwargs):
    """Get or set global configurations."""

    common.bootstrap(ensure_installation=True, print_logo=False)

    try:
        integration.config(**kwargs)
    except error.NoSuchConfigVariable as e:
        error.handle_exception()
        integration.no_such_config_variable(e, is_global=kwargs["is_global"])
