"""General section of a module definition."""

import dataclasses as dc
from typing import Optional, List, Dict

from armarx_setup.core import error
from armarx_setup.core.util import dataclass
from armarx_setup.core.module.context import Hook


@dc.dataclass
class Database(Hook):
    for_namespace: str = ""


@dc.dataclass
class Deprecated(Hook):
    since: str = ""
    removal: Optional[str] = None
    superseded_by: Dict[str, "Dependency"] = None
    message: str = ""


@dc.dataclass
class General(Hook):
    url: Optional[str] = None
    authors: Optional[List[str]] = None
    authors_parsed: Optional[Dict[str, str]] = None
    database: Optional[Database] = None
    deprecated: Optional[Deprecated] = None

    def __post_init__(
            self,
    ):

        self.database = dataclass.sanitize_type(self.database, Database, ctx=self.ctx)
        self.deprecated = dataclass.sanitize_type(self.deprecated, Deprecated, ctx=self.ctx)

        if isinstance(self.authors, str):
            self.authors = [self.authors]

        if self.authors is not None and len(self.authors) > 0:
            from email.utils import parseaddr
            self.authors_parsed = {}
            for author in self.authors:
                display_name, mail_address = parseaddr(author)
                if mail_address == "" or display_name == "":
                    raise error.MalformedAuthorReference(author)
                self.authors_parsed[mail_address] = display_name
