import typing as ty

from .core import JsonData


def remove_items_if(
        data: JsonData,
        remove_dict_key_if: ty.Callable[[str], bool] = None,
        remove_dict_value_if: ty.Callable[[ty.Any], bool] = None,
        remove_list_elem_if: ty.Callable[[ty.Any], bool] = None,
) -> JsonData:
    """
    Remove any dict keys, dict values or list elements for which `predicate` returns True.
    """

    def keep_all(item):
        return False

    if remove_dict_key_if is None:
        remove_dict_key_if = keep_all
    if remove_dict_value_if is None:
        remove_dict_value_if = keep_all
    if remove_list_elem_if is None:
        remove_list_elem_if = keep_all

    kwargs = dict(
        remove_dict_key_if=remove_dict_key_if,
        remove_dict_value_if=remove_dict_value_if,
        remove_list_elem_if=remove_list_elem_if,
    )

    if isinstance(data, dict):
        return {
            key: remove_items_if(value, **kwargs)
            for key, value in data.items()
            if not (remove_dict_key_if(key) or remove_dict_value_if(value))
        }
    elif isinstance(data, list):
        return [
            remove_items_if(item, **kwargs)
            for item in data
            if not remove_list_elem_if(item)
        ]
    else:
        return data
