import dataclasses as dc
import typing as ty

from .core import JsonData
from .switch import resolve_switch


@dc.dataclass
class ArchInfo:
    # e.g. "x86_64"
    architecture_id: str
    # e.g. 32bit / 64bit
    bits: str
    # e.g Linux-5.4.0-135-generic-x86_64-with-Ubuntu-18.04-bionic
    platform: str


# The architecture usually does not change during execution,
# so we can cache the result.
arch_info: ty.Optional[ArchInfo] = None


def get_arch_info() -> ArchInfo:
    global arch_info

    if arch_info is None:
        import platform
        arch_info = ArchInfo(architecture_id=platform.machine(), bits=platform.architecture()[0],
                             platform=platform.platform())

    return arch_info


def resolve_switch_arch(
        data: JsonData,
        arch_name: str = None,
) -> JsonData:
    if arch_name is None:
        arch_name = get_arch_info().architecture_id

    return resolve_switch(data, switch_key="switch_arch", case_name=arch_name)
