#!/bin/bash

set -e

recommended_ccache_max_size="100"  # In GB.

if current_ccache_max_size=$(ccache --get-config max_size 2> /dev/null); then
    # Search and replace "G" with the empty string
    current_ccache_max_size="${current_ccache_max_size/G/}"
else
    # Old versions of ccache didn't had --get-config, so the above command will fail.  This is the fallback.
    current_ccache_max_size=$(ccache --show-stats | grep "max cache size" | awk '{ print $4 }')
fi
current_ccache_max_size="${current_ccache_max_size%.*}"  # float to int.

if axii_config_ccache_max_size=$(axii config --global ccache_max_size); then
    if [ "$current_ccache_max_size" -lt "$axii_config_ccache_max_size" ]; then
        echo "Setting Cache maximum cache size to $axii_config_ccache_max_size GB as per Axii"\
             "config by running:"
        echo ""
        echo "    ccache --max-size ${axii_config_ccache_max_size}GB"
        echo ""
        ccache --max-size ${axii_config_ccache_max_size}GB
    fi

    # If user set a value via `axii config`, stop here.
    echo "Ccache maximum cache size is configured to >=$axii_config_ccache_max_size GB as per Axii config."
    exit 0
fi

if [ "$current_ccache_max_size" -lt "$recommended_ccache_max_size" ]; then
    echo "The recommended maximum cache size for Ccache is $recommended_ccache_max_size GB,"\
         "however Ccache is currently configured to only use $current_ccache_max_size GB at"\
         "most. Please supply Axii with the maximum cache size with the following command:"
    echo ""
    echo "    axii config --global ccache_max_size $recommended_ccache_max_size"
    echo ""
    echo "If you are certain that a maximum cache size of $current_ccache_max_size GB is"\
         "sufficient, you can ignore this warning with the following command:"
    echo ""
    echo "    axii config --global ccache_max_size $current_ccache_max_size"
    echo ""
    echo "Please run one of the above commands to resolve this issue and re-run the upgrade."
    exit 1
else
    echo "Ccache maximum cache size is properly configured to >=$recommended_ccache_max_size GB."
fi
