# Advanced Users' Documentation

Documentation for advanced usage of Axii for increased productivity and sophisticated
use-cases.

[[_TOC_]]

---

## Command Shortcuts

In order to work more efficiently with Axii, you can just type the beginning of a
subcommand, omitting any characters, and Axii will still be able to resolve it as 
long as it is unambiguous. For example:

| Abbreviated command  | Status | Resolves to ...                                      |
|----------------------|--------|------------------------------------------------------|
| `axii w --help`      | ✔️     | `axii workspace --help`                              | 
| `axii w u`           | ❌      | `axii workspace update` or `axii workspace upgrade`? |
| `axii w up`          | ✔️     | `axii workspace update`                              |
| `axii w ug`          | ✔️     | `axii workspace upgrade`                             |
| `axii w g s`         | ✔️     | `axii workspace git status`                          |

The subcommand `axii w u`, is ambiguous, since it may map to `axii workspace update` 
or `axii workspace upgrade`. However, it is enough to supply any characters which make
it unambiguous. For example `axii w ug` will resolve to `axii workspace upgrade` since 
"update" does not contain any "g".


## Advanced Workspace Management

### Default Workspace

You can designate a default workspace Axii should activate each time you open a new 
terminal by running:

```bash
axii config --global default_workspace <your-chosen-workspace-name>
```

You can undo this by running:

```bash
axii config --global --unset default_workspace
```

This way, Axii will not change your environment at all whenever you launch a new
terminal. This is handy when you are working with several workspaces.


### Activating and Deactivating a Workspace

If you don't have a designated default workspace configured, Axii will not change 
anything in your environment. If you want to work with your workspace, you have to 
manually activate it by running:

```bash
axii workspace activate <your-chosen-workspace-name>
```

If you want to deactivate your workspace after working with it, you can run:

```bash
axii workspace deactivate
```

You can see which workspace is active by running:

```bash
axii workspace which
```

Or list the workspaces known to Axii by running:

```bash
axii workspace list
```


### Git Remote Preferences

By default, Axii will clone all repositories using HTTPS as it does not require any additional
setup and is better suited for a quick start.  However, you might want to use SSH at some point
to also push to remote repositories.

For this, Axii features per-host remote preference configurations.  The syntax is:

```shell
axii config --global git_remote_preferences.<Any code hoster URL> [https|ssh]
```

Recommended settings are:

```shell
# Use SSH for H2T GitLab if your public SSH key is configured with it.
axii config --global git_remote_preferences.git.h2t.iar.kit.edu ssh
# Use SSH for GitLab.com if your public SSH key is configured with it.
axii config --global git_remote_preferences.gitlab.com ssh
# Use SSH for GitHub.com if your public SSH key is configured with it.
axii config --global git_remote_preferences.github.com ssh
# Use SSH for KIT/SCC GitLab if your public SSH key is configured with it.
axii config --global git_remote_preferences.git.scc.kit.edu ssh
```

To apply your preferences, run this afterwards and follow the instructions:

```shell
axii workspace git reconfigure
```

To change your preference for a given host to HTTPS, run:

```shell
axii config --global git_remote_preferences.<Code hoster URL> https
axii workspace git reconfigure
```

To restore the defaults for a given host, run:

```shell
axii config --global --unset git_remote_preferences.<Code hoster URL>
axii workspace git reconfigure
```

To restore all defaults, run:

```shell
axii config --global --unset git_remote_preferences
axii workspace git reconfigure
```

**Links:**

- [Configure your SSH keys on H2T GitLab](https://git.h2t.iar.kit.edu/-/profile/keys)
- [Configure your SSH keys on GitLab.com](https://gitlab.com/-/profile/keys)
- [Configure your SSH keys on GitHub](https://github.com/settings/keys)


### Workspace Hygiene

Axii provides several utilities in order to maintain workspaces in the longer term.
It is recommended to run the following subcommand once in a while:

```bash
axii workspace verify
```

This will run all of Axiis built-in verification subcommands, inform you about any
further (interactive) verification and cleanup subcommands, or directly list the 
commands you should execute to fix the corresponding issues.

The remainder of this subsection will describe these dedicated subcommands in detail.


#### Delete Obsolete Modules

Axii will not delete anything in your workspace by its own. This, however, can become
an issue since some CMake project might still find a stray module and thus depend on
a project no longer maintained by Axii.

To detect and delete stray modules, run:

```bash
axii workspace auto-remove
```

Axii will then analyize the workspace, and suggest to remove all paths which are not
actively maintained by Axii.


#### Git Remote Settings

Your Git remote URL preferences, or the URLs of Axii's modules may change from time to
time, especially when dependencies need to be forked or projects are migrated.

To ensure that the local Git repository settings match those in Axii's module database 
and your remote preferences, run:

```bash
axii workspace git reconfigure
```

Use the `--files` flag to additionally get a list of all changed/untracked files on a
per-repository basis.


#### Local Git Changes

Axii configures all Git repositories as defined in the module definitions. When working
with Git repositories, however, you will naturally change branches, have uncommitted 
changes, etc.

To ease working with Git repertoires, especially to show uncommitted changes and
differences to Axii's module definitions, run:

```bash
axii workspace git status
```


## Troubleshooting Guide for Workspace Upgrades

Axii cannot guarantee that upgrades will always succeed, especially if partial upgrades or manual
`prepare`s or `build`s are performed for time saving reasons.  However, Axii gives you i) all
necessary information to debug the problems yourselves and ii) tools and commands to correct the errors.
The necessary information is provided by Axii printing the full commands with all arguments, as 
well as the complete output of these commands for you to inspect, copy & paste and run yourself,
fiddle around with it, and eventually incorporate any necessary changes into the module 
definitions and provide the changes for everyone to benefit from it.  The necessary tools are 
provided by several subcommands of Axii which will be elaborated in the following.

The best general troubleshooting routine (in order of least time consuming to "it's home time!")
proved to be the following.  After each step, you may try it again.  If it still doesn't successfully
upgrade, proceed with the next step.

1. Make sure you are on the Axii stable release branch: `axii self change-release stable`
2. Make sure that Axii is up-to-date: `axii self update`
3. Verify the integrity of your workspace with `axii workspace verify`; follow the orders to
  fix potential issues and try it again with a full `axii workspace upgrade`.
  - Especially if there are any stray modules Axii is not actively maintaining anymore it might
    become dangerous, since these might still be found via CMake, and producing compile errors or
    segmentation faults if not built correctly.
4. Clean the modules `build` folder (`axii workspace clean -tn` in the failing module in your 
  workspace) or Python `.venv` folder (by deleting it manually), depending on the concrete problem.
  You can try it again with a local `axii workspace upgrade -tn` or a full `axii workspace upgrade`
  if need be.
5. **Beware of data loss** Delete the whole module in your workspace and set it up again with a full
  `axii workspace upgrade`.
  - Uncommitted / unpushed changes will be lost, so push them before doing that.
6. Clean all build artefacts in the whole workspace with `axii workspace clean` and try a full clean
  build again with `axii workspace upgrade`.
7. Set up a new temporary workspace with `axii workspace create` and test it again there.

If the problem still persists, it is very likely not a temporary problem or misconfiguration resulting
from partial upgrades or similar.  What you can still do:
- Check Axii's [issue board](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/issues) if the issue 
  has already been reported
- [Open a new issue](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/issues/new) and use a 
  corresponding issue template to provide all necessary information
- Maybe the issue was already fixed on Axii's development branch `main`.  Change to Axii's development
  branch using `axii self change-release main` and start again from step 2. above.
- Dive into the problem, study and understand it, and try to fix the problematic module definition.
  Open a merge request to share fixes with the whole team.


## Building and Upgrading Workspaces

### Cooperative Build Job Numbers

There are three additional special "cooperative" values for `build_job_number`. They behave like their 
"non-cooperative" counterparts, but additionally subtract the ceiled 1-minute load average of the system 
(minimum 1).

```sh
# Use all cores of the current system (cooperative, minimum 1):
axii config --global build_job_number all_coop
# Use all cores of the current system, but one (cooperative, minimum 1):
axii config --global build_job_number all_but_one_coop
# Use half the cores of the current system (cooperative, minimum 1):
axii config --global build_job_number half_coop
```

These settings can be useful if your system has an innegligible background workload.


### Override Build Job Number for a Specific Workspace

You can explicitly override the build job number Axii uses for a specific workspace by setting
`CMAKE_BUILD_PARALLEL_LEVEL` in the workspace's `armarx-workspace.json`: 

```json
{
  "global": {
    "build": {
      "cmake": {
        "env": {
          "CMAKE_BUILD_PARALLEL_LEVEL": "7"
        }
      }
    }
  }
}
```


## Extend the Built-In Module Database

The built-in module definitions are stored in [`data/modules`](data/modules). 
A module name is the path to a `.json` file (the module definition) 
relative to the root of the module database 
(see the [Module Author's Documentation](../module_authors) for more details).

You can extend the built-in module database by setting the environment variable `ARMARX_MODULES`, 
which is a colon (`:`) separated list of root directories in which to look for modules
(similar to how you can modify the `PATH` variable to find executables).
This can be done by adding lines like this to the `~/.bashrc`:

```bash
export ARMARX_MODULES="$HOME/armarx-modules:$ARMARX_MODULES"
```


## Advanced Axii Commands Worth Aliasing

### Open Git GUI in Repositories With Uncommitted Changes

```shell
# Iterates workspace, and opens `git gui` in every module with uncommitted changes along dependency chain.
alias ws_git_gui='axii w exec --git "test -z \"\`git status --porcelain\`\" || git gui"'
```


## Tool Integrations

This section documents how a variety of tools can be integrated in and installed via Axii.

Generally it is considered best practice to create a dedicated **tools workspace** 
to contain tools installed via Axii (e.g., IDEs), so they can be upgraded separately.

This is especially suitable for modules in `tools/`, e.g. IDEs, build tools, and 
specific Python versions.
These modules are workspace-aware (or at least independent), so you can run them
from any activated workspace.

To do so, create a tools workspace called "tools":
```shell
axii workspace create ~/tools tools
axii workspace activate tools
```

Remove modules that were added by default:
```shell
axii workspace remove armarx
axii workspace remove h2t/code_style
```

Add your desired tool modules:
```shell
axii workspace add tools/...  # Use autocompletion or see below for available modules
```

Finally run an upgrade as usual:
```shell
axii workspace upgrade
```

To make the tool commands available without activating the tools workspace,
add the binary folder (`.axii/bin`) of the tools workspace to your `PATH` variable
in your `~/.bashrc` or `~/.zshrc`:

```bash
export PATH="~/tools/.axii/bin:$PATH"
```

To add this line, you can run (**with the tools workspace activated**):
```shell
echo "export PATH=$ARMARX_WORKSPACE/.axii/bin:\$PATH" >> ~/.bashrc
```


### Build Tools

#### IDE's

##### QtCreator

Axii can be used to fully install and configure QtCreator. This configuration includes
fully preconfigured kits, preconfigured code-style tools (clang-format, clang-tidy), as
well as workspace-aware user-configurations.

- Add module:
  ```bash
  axii workspace add tools/qtcreator/5.0  # For bionic (Ubuntu 18), or ...
  axii workspace add tools/qtcreator/10.0  # ... for focal, jammy (Ubuntu 20, 22)
  ```
- Install:
  ```bash
  axii workspace upgrade
  ```
- Start:
  ```bash
  qtcreator-5.0  # For bionic, or ...
  qtcreator-10.0  # ... for focal, jammy
  ```

See the the [ArmarX "Hello World!" tutorial (Open the ArmarX Package in QtCreator)](https://armarx.humanoids.kit.edu/tutorials_100_hello_world_cpp.html#autotoc_md82) for how to open an ArmarX package with QtCreator.


##### PyCharm

There are modules for **community** or **professional** editions of PyCharm.
For professional editions, you will need a license (you can apply for an educational license [here](https://www.jetbrains.com/community/education/#students)).
The community edition is free to use.

- Add module:
  ```bash
  axii workspace add tools/pycharm/community/2023.1  # Free community edition
  axii workspace add tools/pycharm/professional/2023.1  # Professional, requires a license
  ```
- Install:
  ```bash
  axii workspace upgrade
  ```
- Start:
  ```bash
  pycharm-community-2023.1  # Community edition
  pycharm-professional-2023.1  # Professional edition
  ```

Later, feel free to update PyCharm further using its on-board tools.


##### VS Code

Axii provides the latest VS Code version alongside with some useful extensions and a default, workspace-aware configuration. This allows you to directly start coding without the need to setup anything manually.

All you need to do is:

- Add module:
  ```bash
  axii workspace add tools/vs_code
  ```
- Install:
  ```bash
  axii workspace upgrade
  ```
- Start:
  ```bash
  code
  ```

During first execution of `code` in a workspace, Axii will copy configuration files (settings, keybindings and tasks) to `$ARMARX_WORKSPACE/.axii/config/vs_code`.
Installing these files can also be manually invoked by running `code_configure` at any time to reset the settings to Axii's default.

Useful commands are as follows (inspired by QtCreator):

| Shortcut  | Action  |
|---|---|
| Ctrl + B  | Builds the Axii module associated with the current file.  |
| Ctrl + P  | Search for file |
| Ctrl + T | Search for symbols (function, variables, ...)  |
| F2  |  Follow symbol under cursor  |
| F4  |  Toggle header/source |
| Ctrl + Shift + I  |  Format the current file  |


#### CMake

Axii rolls out its own version of CMake as soon as your workspace requires any CMake project or C++ IDE.


##### CMake Build Type

For easier debugging, set the CMake build type to `RelWithDebInfo` in your
`armarx-workspace.json`:

```json
{
  "global": {
    "prepare": {
      "cmake": {
        "definitions": {
          "CMAKE_BUILD_TYPE": "RelWithDebInfo"
        }
      }
    }
  }
}
```


##### Disable CMake's Global Registry

Some projects, such as all ArmarX packages, might register themselves in the global
CMake registry found in `~/.cmake/packages`. This however violates the concept of 
isolated workspaces. All modules should expose the CMake project they contain by 
providing a `project_DIR` environment variable. Therefore, it is advised to disable 
the global registration in your `armarx-workspace.json` as follows:

```json
{
  "global": {
    "prepare": {
      "cmake": {
        "definitions": {
          "CMAKE_EXPORT_NO_PACKAGE_REGISTRY": "ON"
        }
      }
    }
  }
}
```


#### Ninja

To use Ninja as your default CMake generator, simply add the `tools/ninja/default`
module to your `armarx-workspace.json`, and configure your generator as follows:

```json
{
  "modules": {
    "tools/ninja/default": {},
    ...
  },
  
  "global": {
    "prepare": {
      "cmake": {
        "generator": {
          "name": "Ninja",
          "args": "-k9999"
        }
      }
    }
  }
}
```

Finally, run `axii workspace upgrade`.


##### Colored Output

The compiler output won't be colored by default if `ninja` is used. To enable it, 
adjust your `armarx-workspace.json` as follows:

```json
{
  "modules": {
    "tools/ninja/default": {},
    ...
  },
  
  "global": {
    "prepare": {
      "cmake": {
        "definitions": {
          "FORCE_COLORED_OUTPUT": "ON",
          "ARMARX_CMAKE_ADDON_FORCE_COLORED_OUTPUT": "ON"
        }
      }
    }
  }
}
```


#### Ccache

To make use of Ccache, add the `tools/ccache/default` module to export the `$CCACHE` 
variable, and then set the corresponding CMake variables in your 
`armarx-workspace.json` as follows:

```json
{
  "modules": {
    "tools/ccache/default": {},
    ...
  },

  "global": {
    "prepare": {
      "cmake": {
        "definitions": {
          "CMAKE_C_COMPILER_LAUNCHER": "$CCACHE",
          "CMAKE_CXX_COMPILER_LAUNCHER": "$CCACHE"
        }
      }
    }
  }
}
```

Finally, run `axii workspace upgrade`.

By default, the default cache size of Cache is 5GB. Generally, however, a cache size
of 100GB is recommended for ArmarX workspaces. To do so, run:

```bash
ccache -M 100G
```


### Other Tools

#### Autojump

If you use [autojump](https://github.com/wting/autojump) 
and want to inform autojump about the module directories, you can run: 
```bash
axii workspace exec "autorun -i 1"
```
