# Developers' Documentation

Documentation for Axii developers.

[[_TOC_]]

---

## General and Prerequisites

Developers of Axii should install all development dependencies. These are required for testing. To do so, run:

```bash
# In Axii's root directory.
./bootstrap --apt --dev
```


## Testing

To run all tests, including unit tests and end-to-end (E2E) tests, run:

```bash
# In Axii's root directory.
.venv/bin/pytest tests
```


### Unit Testing

You can run all unit tests as follows:

```bash
# In Axii's root directory.
.venv/bin/pytest tests/unit
```


### End-to-End Testing

You can run all E2E tests as follows:

```bash
# In Axii's root directory.
.venv/bin/pytest tests/e2e
```


## CI Command Line Interface

The `axii ci` subcommand groups commands that can be used by a CI pipeline or related tools.


### List the Apt Packages Required by the Workspace

To list the apt packages required by the workspace in a format that can be passed to `apt-get install`,
use:
```shell
axii ci workspace list-apt-packages
```

Examples:

- List **all** required apt packages:
    ```shell
  $ axii ci workspace list-apt-packages --all
  libbz2-dev automake protobuf-compiler pulseaudio-utils mpg123 pkg-config libxrandr-dev libtiff-dev git autoconf libopengl-dev libomp5 
  python3-argcomplete libmongoclient-dev bison libarmadillo-dev git-lfs jq gcc-9 build-essential libatlas-base-dev libtcl8.6 
  libprotobuf-dev libmpfr-dev libomp-dev libtk8.6 libqt5designer5 libqt5designercomponents5 ccache g++-9 libgmp-dev libalglib-dev 
  zeroc-ice-all-dev libglew-dev libicu-dev liburdfdom-dev libssh-dev swig libgflags-dev libffi-dev liblz4-dev qtbase5-dev libboost-all-dev 
  qtconnectivity5-dev freeglut3-dev screen libglib2.0-dev libdc1394-22-dev qttools5-dev libassimp-dev google-mock liblmdb-dev pulseaudio 
  libssl-dev libqt5bluetooth5 libgoogle-glog-dev libqwt-qt5-dev libsuitesparse-dev python3-venv libblas-dev libqhull-dev libnlopt-cxx-dev 
  curl python3-setuptools libraw1394-dev libgsl-dev libeigen3-dev qtbase5-dev-tools libpcl-dev qtscript5-dev qt5-qmake libgraphviz-dev 
  libjpeg-dev libbson-dev python3 libqt5svg5-dev flac qtchooser python3-dev liblmdb++-dev libjsoncpp-dev zeroc-ice-all-runtime 
  libminizip-dev libsqlite3-dev libtiff-tools libfftw3-dev libnlopt-dev liblapack-dev libfann-dev libmongoc-dev libtool portaudio19-dev 
  libvtk6.3
  ```

- Missing apt packages (on my machine):
  ```shell
  $ echo "Missing: '$(axii ci workspace list-apt-packages --missing)'"
  Missing: ''
  ```


## Coding Conventions

### Exception Classes

Exception classes raised by Axii are defined in 
[`armarx_setup/core/error.py`](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/blob/main/armarx_setup/core/error.py).
The common base class of all exceptions is `error.AxiiError`.

When raising exceptions, always use one of these exception classes or create a new one if none is suitable. 
Names of exception classes should name their cause, i.e. what went wrong.
They should avoid the term "error", as this is indicated by the module `error` already.


### Exception Handling

In each `except` block, use `error.handle_exception()` (see !431):

```python
try:
    something_which_might_fail()
except error.ConcreteError:
    error.handle_exception()
    other_error_handling()
```

Optionally, a reason can be supplied why this exception type is expected using the argument
`expected_because` of `error.handle_exception()`:

```python
    ...
except error.ConcreteError:
    error.handle_exception("This can happen because ...")
    ...
```
