# Maintainers' Documentation

Documentation for Axii maintainers.

[[_TOC_]]

---


## Review Merge Requests

See the
[contributing documentation](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/blob/main/CONTRIBUTING.md?ref_type=heads#when-you-think-your-contribution-is-finished)
for descriptions. If there is a merge request without any type, assign one.


## Versioning

Axii is versioned as follows: `yy.mm.r.p`, with:

- `yy` being the two-digit year
- `mm` being the two-digit month
- `r` being the release number of the month, beginning with "1"
- `p` being the patch number for patch/hotfix releases


## Releases

Releases are fully automated by Axii itself via a subcommand.  Thus, creating a release requires
certain access rights, which are provided to Axii by a Personal Access Token (PAT) of a Gitlab
user with maintainer or owner rights.


### Setup and Prerequisites

The following steps need to be done only once, when getting maintainer status and being about to
do the first release, or after a new installation of axii.


#### Setup Personal Access Token in GitLab

To trigger a release, you must be a maintainer of Axii (owner role in GitLab)
and Axii must be configured to have an API-level Personal Access Token (PAT). To do so:

- [Generate a PAT](https://git.h2t.iar.kit.edu/-/profile/personal_access_tokens)
  - Name it "Axii" (or whatever you like)
  - Check "api", the rest may stay unchecked
  - Unset the expiry date
  - Copy the PAT to your clipboard
- Run `axii config --global axii_gitlab_pat <paste PAT>` to configure Axii and set
the PAT


#### Maintainer Information

The maintaining persons occur in auto-generated text of the relase announcement e-mail and its receivers.
Thus, if the group of maintainers changes, adapt the information stored in
[.gitlab/CODEOWNERS](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/blob/main/.gitlab/CODEOWNERS).


#### Install Development Packages

- Ensure that the development dependencies are installed, by running the following in your Axii
  repository root:
  ```shell
  ./bootstrap.sh --dev --apt
  ```


### For Each Release

- **Major release only**: Ensure that the [milestone corresponding to the version](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/milestones)
about to be released is ready:
  - The name must match the version to be released.
  - There must be no open merge request or issue.
  - Assure that all
    [closed issues for this release](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/issues/?sort=created_date&state=closed)
    are assigned to the milestone to be released
  - Assure that all
    [closed merge requests for this release](https://git.h2t.iar.kit.edu/sw/armarx/meta/axii/-/merge_requests?scope=all&state=merged)
    are assigned to the milestone to be released
  - Assure that the merge request titles are self-contained, as they will end up as changelog entry.
  - Assure that `type::` tags are assigned properly, as they will be used for the changelog.
    See [how to review](#how-to-review)
  - Assure that assignees are set correctly.
- Ensure that there are no uncommitted changes.
- Ensure your local state is up-to-date.
  ```shell
  axii self change-release stable
  axii self update
  axii self change-release main
  axii self update
  ```
- Ensure that all tests still pass
  ```shell
  source .venv/bin/activate
  pytest tests
  ```
- Trigger release
  ```shell
  axii self maintain release
  ```
  - You will be asked to bump the version
    - Axii's main module will open
    - Adjust the version accordingly
    - Save the file and close the editor
  - You will be asked to append the changelog
    - Axii's changelog will open with the changelog pre-populated from GitLab
    - Review the merge changelog
    - You can add contributors here, merge changelog entries, etc.
    - Save the file and close the editor
  - You will be asked to review the diff again
    - Ensure that the changes are as expected
    - Type "yes" to automatically create the commits and push
  - Wait for Axii to perform the release and for all pipelines to pass
- After the release
  - Axii provides you a suggestion for a message to announce the release.
    Feel free to distribute it via suitable channels, in case of important
    features or breaking changes.
