from armarx_setup.core.module import Module
from armarx_setup.core.module.common import SubStepAdapter
from armarx_setup.core.module.update import Update, Download

from tests.unit.core.module import DummyContext


def test_default_construction():
    json = {}

    m = Module(**json, name="test_module")

    assert m.name == "test_module"

    assert m.general is None

    assert m.prepare is None
    assert m.system is None
    assert m.update is None
    assert m.prepare is None
    assert m.build is None
    assert m.install is None
    assert m.conclude is None

    assert m.required_modules == {}
    assert m.features == {}


def test_single_update_with_single_download():
    json = {
        "update": {
            "download": {
                "url": "https://example.com/some-file.txt"
            }
        }
    }

    m = Module(**json, name="test_module", ctx=DummyContext())

    assert type(m.update) is Update
    assert type(m.update.download) is Download
    assert m.update.download.url == "https://example.com/some-file.txt"


def test_single_update_with_multiple_downloads():
    json = {
        "update": {
            "download": [
            {
                "url": "https://example.com/some-file.txt"
            }, {
                "url": "http://example.com/generic-download-path",
                "file": "downloaded-file.zip"
            }]
        }
    }

    m = Module(**json, name="test_module", ctx=DummyContext())

    assert type(m.update) is Update
    assert type(m.update.download) is SubStepAdapter
    assert len(m.update.download.sub_steps) == 2
    assert m.update.download.sub_steps[0].url == "https://example.com/some-file.txt"
    assert m.update.download.sub_steps[0].file == "some-file.txt"
    assert m.update.download.sub_steps[1].url == "http://example.com/generic-download-path"
    assert m.update.download.sub_steps[1].file == "downloaded-file.zip"


def test_multiple_updates_with_single_download():
    json = {
        "update": [
            {
                "download": {
                    "url": "https://example.com/some-file.txt"
                }
            }, {
                "download": {
                    "url": "http://example.com/generic-download-path",
                    "file": "downloaded-file.zip"
                }
            }
        ]
    }

    m = Module(**json, name="test_module", ctx=DummyContext())

    assert type(m.update) is SubStepAdapter
    assert len(m.update.sub_steps) == 2
    assert type(m.update.sub_steps[0]) is Update
    assert type(m.update.sub_steps[1]) is Update
    assert m.update.sub_steps[0].download.url == "https://example.com/some-file.txt"
    assert m.update.sub_steps[0].download.file == "some-file.txt"
    assert m.update.sub_steps[1].download.url == "http://example.com/generic-download-path"
    assert m.update.sub_steps[1].download.file == "downloaded-file.zip"
