from armarx_setup.core.util.json.remove import remove_items_if


def test_remove_list_items():
    out = remove_items_if(["a", "B", "c", "D"], remove_list_elem_if=lambda s: s.lower() != s)
    expected = ["a", "c"]
    assert out == expected, f"out: \n{out}\n expected: \n{expected}"


def test_remove_nested_list_items():
    out = remove_items_if({
        "items": ["a", "B", "c", "D"],
        "others": {
            "items2": ["A", "b", "C", "d"],
            "empty": [],
        }
    }, remove_list_elem_if=lambda s: s.lower() == s)
    expected = {
        "items": ["B", "D"],
        "others": {
            "items2": ["A", "C"],
            "empty": [],
        }
    }
    assert out == expected, f"out: \n{out}\n expected: \n{expected}"


def test_remove_dict_keys():
    out = remove_items_if({
        "one": 1,
        2: "two",
        "three": 3,
        4: "four",
    }, remove_dict_key_if=lambda k: isinstance(k, str))
    expected = {
        2: "two",
        4: "four",
    }
    assert out == expected, f"out: \n{out}\n expected: \n{expected}"


def test_remove_nested_dict_keys():
    out = remove_items_if({
        0: ["discarded"],
        "items": {
            "one": 1,
            2: "two",
            "three": 3,
            4: "four",
        },
        "deep": {
            "items2": {
                42: "fourty-two",
                "answer": 42,
            }
        }
    }, remove_dict_key_if=lambda k: isinstance(k, int))
    expected = {
        "items": {
            "one": 1,
            "three": 3,
        },
        "deep": {
            "items2": {
                "answer": 42,
            }
        }
    }
    assert out == expected, f"out: \n{out}\n expected: \n{expected}"


def test_remove_dict_values():
    out = remove_items_if({
        "one": 1,
        "five": 5,
        "ten": 10,
        "fifty": 50,
    }, remove_dict_value_if=lambda v: v >= 10)
    expected = {
        "one": 1,
        "five": 5,
    }
    assert out == expected, f"out: \n{out}\n expected: \n{expected}"


def test_remove_nested_dict_values():
    out = remove_items_if({
        "removed": ["list"],
        "items": {
            "int": 1,
            "dict": dict(),
            "list": list(),
        },
        "deep": {
            "items2": {
                "int": 1,
                "dict": dict(),
                "list": list(),
            },
            "also list": [1, 2, 3],
        },
    }, remove_dict_value_if=lambda v: isinstance(v, list))
    expected = {
        "items": {
            "int": 1,
            "dict": dict(),
        },
        "deep": {
            "items2": {
                "int": 1,
                "dict": dict(),
            }
        },
    }
    assert out == expected, f"out: \n{out}\n expected: \n{expected}"
