import copy
import typing as ty


class DummyData:

    def __init__(self):

        raw: ty.Dict[str, ty.Any] = {
            "one": "1",
            "two": {
                "axii.switch_arch": {
                    "default": "2",
                    "x86_64": "x86_2",
                    "aarch64": "arch2",
                    "ppc": "ppc2"
                },
            },
            "three": 3,
            "list": [
                "first",
                {
                    "axii.switch_arch": {
                        "default": "second",
                        "x86_64": "x86_second",
                    }
                },
                "third",
                {
                    "axii.switch_arch": {
                        "default": "fourth",
                        "aarch64": "arch64_fourth"
                    }
                }
            ],
            "nested": {
                "axii.switch_arch": {
                    "default": {
                        "a": "A",
                        "b": {
                            "axii.switch_arch": {
                                "default": "default_default_b",
                                "x86_64": "default_x86_b"
                            }
                        }
                    },
                    "aarch64": "jammy_nested",
                }
            },
            "optional_keys": {
                "always": {},
                "only_x86": {
                    "axii.switch_arch": {
                        "x86_64": {},
                        "default": None
                    }
                },
                "not_on_x86": {
                    "axii.switch_arch": {
                        "x86_64": None,
                        "default": {}
                    }
                }
            },
            "optional_list_entries": [
                "always",
                {
                    "axii.switch_arch": {
                        "x86_64": "only_x86",
                        "default": None
                    }
                },
                {
                    "axii.switch_arch": {
                        "x86_64": None,
                        "default": "only_default"
                    }
                },
                "always 2"
            ]
        }


        expected_default = copy.deepcopy(raw)
        expected_default["two"] = "2"
        expected_default["list"][1] = "second"
        expected_default["list"][3] = "fourth"
        expected_default["nested"] = {
            "a": "A",
            "b": "default_default_b"
        }
        del expected_default["optional_keys"]["only_x86"]
        expected_default["optional_keys"]["not_on_x86"] = {}
        expected_default["optional_list_entries"][2] = "only_default"
        del expected_default["optional_list_entries"][1]

        expected_x86 = copy.deepcopy(raw)
        expected_x86["two"] = "x86_2"
        expected_x86["list"][1] = "x86_second"
        expected_x86["list"][3] = "fourth"
        expected_x86["nested"] = {
            "a": "A",
            "b": "default_x86_b"
        }
        expected_x86["optional_keys"]["only_x86"] = {}
        del expected_x86["optional_keys"]["not_on_x86"]
        expected_x86["optional_list_entries"][1] = "only_x86"
        del expected_x86["optional_list_entries"][2]

        self.raw = raw
        self.expected = {
            "default": expected_default,
            "x86_64": expected_x86,
        }


def test_json_resolve_switch_arch():
    from armarx_setup.core.util.json.switch_arch import resolve_switch_arch

    td = DummyData()

    for arch_name, expected in td.expected.items():
        got = resolve_switch_arch(data=td.raw, arch_name=arch_name)
        assert got == expected
